#include "RACE_LogfileOffset.h"
#include "RACE_Accumulator.h"
#include "envdebug.h"
#include "output.h"

static envdebug *debug = new envdebug("RACE_Accumulator");

//##ModelId=3F72A5C50302
RACE_AccumulatorObserver::~RACE_AccumulatorObserver()
{
}

//##ModelId=3F72A5C502BC
RACE_Accumulator::RACE_Accumulator(RACE_WriterInterface *writer,size_t size,size_t blocksize):m_Writer(writer)
{
  mp_entry = new SEntry[size];
  RACE_COND_FAIL(mp_entry==NULL,"%s:%d: RACE_Accumulator: No more Memory",__FILE__,__LINE__);
  m_io = new iovec[size];
  RACE_COND_FAIL(m_io==NULL,"%s:%d: RACE_Accumulator: No more Memory",__FILE__,__LINE__);
  m_CurrentOffset = 0;
  m_CurrentSize = 0;
  m_Index=0;
  m_IndexMax=size;
  m_BlockSize=blocksize;
  m_AccuObserver=0;
}

//##ModelId=3F72A5C502CC
RACE_Accumulator::~RACE_Accumulator()
{
  delete[] mp_entry;
  delete[] m_io;
}

//##ModelId=3F72A5C502CD
void RACE_Accumulator::Add(RACE_Dumpable *d)
{
  if(d->IsFragmented())
  {
    debug->print("RACE_Accumulator::add: is_fragmented");
    int i=0;
    // The dumpable is released in the iterator, so for iterating
    // we have to duplicate the dumpable.
    // At the end all is deleted
    for(d->begin();!d->is_done();d->advance())
    {
      RACE_Dumpable *d2 = ACE_dynamic_cast(RACE_Dumpable *,d->duplicate());
      debug->print("RACE_Accumulator::add: fragment length %d/%d",
                   d2->length(),d2->total_length());
      AddFrag(d2,i>0);
      i++;
    }
    d->release();
  }
  else
  {
    // Optimization for the common non fragmented case
    debug->print("RACE_Accumulator::add: length %d/%d",
                   d->length(),d->total_length());
    AddFrag(d,false);
  }
}

//##ModelId=3F72A5C502EB
void RACE_Accumulator::AddFrag(RACE_Dumpable *d,bool frag)
{
  size_t nw;
  int curindex;
  if(m_Index>=m_IndexMax)
  {
    Flush();
  }
  debug->print("RACE_Accumulator::AddFrag: currentsize=%ld add %ld",
                 m_CurrentSize,d->length());
  curindex=m_Index;
  mp_entry[m_Index].mp_dumpable=d;
  mp_entry[m_Index].m_offset=m_CurrentOffset;
  mp_entry[m_Index].m_notify=!frag;  // true normaly, fragmented only notified once
  m_io[m_Index].iov_base=d->rd_ptr();
  m_io[m_Index].iov_len= d->length();
  m_CurrentOffset += d->length();
  m_Index++;
  while(mp_entry[curindex].mp_dumpable != NULL
        && m_CurrentSize+mp_entry[curindex].mp_dumpable->remaining() >= m_BlockSize)
  {
    m_io[curindex].iov_len=m_BlockSize-m_CurrentSize;
    mp_entry[curindex].mp_dumpable->rd_ptr(m_io[curindex].iov_len);
    nw=m_Writer->WriteV(m_io,m_Index);
    debug->print("RACE_Accumulator::add: Writing %d bytes",nw);
    Notify();
    RACE_Dumpable *h = mp_entry[curindex].mp_dumpable;
    mp_entry[curindex].mp_dumpable = NULL;
    FlushAfterWrite();
    curindex=0;
    m_io[curindex].iov_base=h->rd_ptr();
    m_io[curindex].iov_len=h->remaining();
    if(h->remaining()>0)
    {
      m_Index=1;
      mp_entry[curindex].mp_dumpable=h;
      mp_entry[curindex].m_notify=false;
    }
    else
    {
      h->release();
    }
  }
  m_CurrentSize += m_io[curindex].iov_len;
}

//##ModelId=3F72A5C502E6
void RACE_Accumulator::FlushAfterWrite()
{
  int i;
  if(m_AccuObserver)
  {
    m_AccuObserver->NotifyFlush();
  }
  for(i=0;i<m_Index;i++)
  {
    if(mp_entry[i].mp_dumpable)
    {
      mp_entry[i].mp_dumpable->release();
      mp_entry[i].mp_dumpable=NULL;
    }
  }
  m_Index=0;
  m_CurrentSize=0;
}

//##ModelId=3F72A5C502E9
void RACE_Accumulator::Notify()
{
  int i;
  for(i=0;i<m_Index;i++)
  {
    if(mp_entry[i].mp_dumpable !=NULL && mp_entry[i].m_notify)
    {
      mp_entry[i].mp_dumpable->InformOffset(RACE_LogfileOffset(m_Writer->PartNo(),
                                                        mp_entry[i].m_offset));
    }
  }
}

//##ModelId=3F72A5C502D6
void RACE_Accumulator::Flush()
{
  size_t nw;
  if(m_Index>0)
  {
    nw=m_Writer->WriteV(m_io,m_Index);
    debug->print("RACE_Accumulator::Flush(): %d Bytes written",nw);
    Notify();
    FlushAfterWrite();
  }
  else
  {
    debug->print("RACE_Accumulator::Flush(): was empty");
  }
}

//##ModelId=3F72A5C502DC
void RACE_Accumulator::LogSwitch()
{
  m_CurrentOffset=0;
}
