#ifndef RACE_ACCUMULATOR_H
#define RACE_ACCUMULATOR_H
#include"RACE_Dumpable.h"
#include<sys/uio.h>

//##ModelId=3F72A5C501B1
//##Documentation
//##  Abstract Interface for Writing a file.
//##  The WriteV - Method uses the writev(2)-Systemcall to gather blocks
//##  togther without copiing

class RACE_WriterInterface
{
public:
    //##ModelId=3FB3861C03B2
  virtual ~RACE_WriterInterface () {}
    //##ModelId=3F72A5C802CB
    //##Documentation
    //## Interface like the Systemcall writev(2)
  virtual int WriteV(const  struct  iovec  *  vector,  int count)=0;
    //##ModelId=3F72A5C90045
    //##Documentation
    //## Returns the number of the current part of the logfile
  virtual long PartNo()=0;
};

//##ModelId=3F72A5C5014E
//##Documentation
//## Abstract Interface for observing the RACE_Accumulator
//## With this Interface a minimal flush rate can implemented
//## To avoid extra synchronisation, we want to send a FlushDumpable
//## But here we don't know what "Sending" is and we must decouple the
//## Implemantation
class RACE_AccumulatorObserver
{
public:
    //##ModelId=3F72A5C50300
  virtual void NotifyFlush()=0;
    //##ModelId=3F72A5C50302
  virtual ~RACE_AccumulatorObserver();
};

//##ModelId=3F72A5C5013D
//##Documentation
//##  The Accumulator accumulates Dumpables to collect systemsize Blocks
//##  for instance  4K-Blocks

class RACE_Accumulator
{
public:
    //##ModelId=3F72A5C502BC
  RACE_Accumulator(RACE_WriterInterface *writer,size_t size=20,size_t blocksize=4096);
    //##ModelId=3F72A5C502CC
  ~RACE_Accumulator();
    //##ModelId=3F72A5C502CD
    //##Documentation
    //## add a Dumpable
  void Add(RACE_Dumpable *d);
    //##ModelId=3F72A5C502D6
    //##Documentation
    //## force the writing the collected blocks to the Writer
  void Flush();
    //##ModelId=3F72A5C502D9
    //##Documentation
    //## get the current offset in the current part
  size_t GetCurrentOffset() { return m_CurrentOffset; }
    //##ModelId=3F72A5C502DC
    //##Documentation
    //## reset the internal state for the switching the logfile
  void LogSwitch();
    //##ModelId=3F72A5C502DE
  void SetAccuObserver(RACE_AccumulatorObserver *obs) { m_AccuObserver=obs; }
protected:
    //##ModelId=3F72A5C502E6
  void FlushAfterWrite();
    //##ModelId=3F72A5C502E9
  void Notify();
    //##ModelId=3F72A5C502EB
  void AddFrag(RACE_Dumpable *d,bool frag=false);

private:
      //##ModelId=3F72A5C50143
  struct SEntry
  {
        //##ModelId=3F72A5C50250
    RACE_Dumpable *mp_dumpable;
        //##ModelId=3F72A5C5025B
        //##Documentation
        //## the offset in the current part
    off_t m_offset;
        //##ModelId=3F72A5C50262
        //##Documentation
        //## whether the dumpable is to be notified
    bool m_notify;
  };
    //##ModelId=3F72A5C5026A
  SEntry        *mp_entry;
    //##ModelId=3F72A5C50275
    //##Documentation
    //## an POSIX-IO-vector see writev(2)
  struct iovec *m_io;

    //##ModelId=3F72A5C5027C
    //##Documentation
    //## The offset in the part
  off_t m_CurrentOffset;
    //##ModelId=3F72A5C50284
    //##Documentation
    //## the current size of the block collected now
  size_t m_CurrentSize;
    //##ModelId=3F72A5C5028C
  int m_Index;
    //##ModelId=3F72A5C50293
  int m_IndexMax;
    //##ModelId=3F72A5C50299
    //##Documentation
    //## The size of the blocks to be written to the Writer
  size_t m_BlockSize;
    //##ModelId=3F72A5C502A1
  RACE_WriterInterface *m_Writer;

    //##ModelId=3F72A5C502AF
  RACE_AccumulatorObserver *m_AccuObserver;
};

#endif
