#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>

#include "RACE_LogIndexPositioner.h"
#include "envdebug.h"
#include "output.h"

static envdebug *s_debug=new envdebug("RACE_LogIndexPositioner");

//##ModelId=3EBA5D420036
RACE_LogIndexPositioner::RACE_LogIndexPositioner(const char *filename)
{
  Open(filename);
}

//##ModelId=3EBA5D42003A
void RACE_LogIndexPositioner::Open(const char *filename)
{
  struct stat sb;
  int r;
  m_fd=::open(filename,O_RDONLY);
  if(m_fd>=0)
  {
  r=fstat(m_fd,&sb);
  RACE_COND_FAIL(r<0,"%s:%d: Cann't fstat %s : %s",__FILE__,__LINE__,
                    filename,strerror(errno));
  mp_entries =(RACE_LogIndexLayout *)mmap(NULL,sb.st_size,PROT_READ,MAP_SHARED,m_fd,0);
  RACE_COND_FAIL(mp_entries<0,"%s:%d: Cann't fstat %s : %s",__FILE__,__LINE__,
                    filename,strerror(errno));
  m_count = sb.st_size/sizeof(RACE_LogIndexLayout);
}
  else
  {
    RACE_ERROR("%s:%d: Cann't Open %s : %s",__FILE__,__LINE__,
                    filename,strerror(errno));
    m_count=0;
    mp_entries = 0;
  }
}

//##ModelId=3EBA5D42003E
RACE_LogIndexPositioner::~RACE_LogIndexPositioner()
{
  munmap((void *)mp_entries,m_count*sizeof(RACE_LogIndexLayout));
  close(m_fd);
}

//##ModelId=3EBA5D42003F
void RACE_LogIndexPositioner::Print()
{
  for(unsigned long i=0;i<m_count;i++)
  {
    struct tm *ts = localtime((time_t *)&(mp_entries[i].unixtime));
    printf("%lu. %d:%.2d:%.2d (%ld) %lu:%lu\n",i,
                  ts->tm_hour,ts->tm_min,ts->tm_sec,
                  mp_entries[i].unixtime,mp_entries[i].offset.part,
                  mp_entries[i].offset.offset);
  }
}

//##ModelId=3EBA5D420043
RACE_LogIndexLayout& RACE_LogIndexPositioner::operator[](unsigned long i) throw (RACE_LogIndexPositioner::RangeError)
{
  if(i<0 || i>=m_count) throw RACE_LogIndexPositioner::RangeError(i,m_count);
  return mp_entries[i];
}

//##ModelId=3EBA5D420047
unsigned long RACE_LogIndexPositioner::GetLastIndex(time_t wst)
{
  unsigned long li=0;
  if(m_count<2 || wst < mp_entries[0].unixtime) return 0;
  if(wst >= mp_entries[m_count-1].unixtime) return m_count-1;
  long T = (mp_entries[m_count-1].unixtime - mp_entries[0].unixtime)/m_count;
  li=(wst-mp_entries[0].unixtime)/T;

  while(li<m_count && mp_entries[li].unixtime < wst) li++;
  while(li>0 && mp_entries[li].unixtime > wst) li--;
  return li;
}

//##ModelId=3EBA5D42004B
time_t RACE_LogIndexPositioner::StrToTime(const char *hhmmss)
{
  time_t ret;
  int r;
  struct tm tmib;
  s_debug->print("StrToTime: First time: %d ",mp_entries[0].unixtime);
  struct tm *tmi=localtime_r(&(mp_entries[0].unixtime),&tmib);
  tmi->tm_hour=0;
  tmi->tm_min=0;
  tmi->tm_sec=0;
  r=sscanf(hhmmss,"%d:%d:%d",&(tmi->tm_hour),&(tmi->tm_min),&(tmi->tm_sec));
  s_debug->print("RACE_LogIndexPositioner::StrToTime: scanned: %d - %d:%d:%d\n",r,
               tmi->tm_hour,tmi->tm_min,tmi->tm_sec);
  //s_debug->print("%d.%d.%d \n",tmi->tm_mday,tmi->tm_mon+1,tmi->tm_year+1900);
  ret=mktime(tmi);
  s_debug->print("time=%d\n",ret);
  return ret;
}
