#ifndef RACE_LOGSWITCHINGSTRATEGY_H
#define RACE_LOGSWITCHINGSTRATEGY_H
#include <sys/types.h>
#include <envdebug.h>
#include "RACE_LogSwitcher.h"

//##ModelId=3F72A5C50194
//##Documentation
//##  There are serveral strategies thinkable, when to begin a new part
//##  of the logfile:
//##  - only when needed at a hard_limit
//##  - when a softlimit is reached and a checkpoint is written
//##  - periodic, for example every hour
//##  This defines the interface of the strategy.
//##  Thus, a special Strategy may be specified even in a logging_script

class RACE_LogSwitchingStrategy
{
public:
    //##ModelId=3FB3861C0335
  virtual ~RACE_LogSwitchingStrategy () {}
    //##ModelId=3F72A5C800BF
  RACE_LogSwitchingStrategy(RACE_LogSwitcher *ls):mp_LogSwitcher(ls),m_Hardlimit(1<<30) {}
    //##ModelId=3F72A5C8018E
  virtual void InfoCurrentOffset(off_t offset);
    //##ModelId=3F72A5C8025E
  virtual void InfoSnapshotWritten()=0;
protected:
    //##ModelId=3FC5CFD701D8
  //##Documentation
  //## All concrete derivations log over this Variable
  static envdebug *ms_debug;
    //##ModelId=3F72A5C70362
  RACE_LogSwitcher *mp_LogSwitcher;
    //##ModelId=3F72A5C7036D
  off_t m_Hardlimit;
    //##ModelId=3F72A5C80024
  off_t m_CurrentOffset;
};

//##ModelId=3F72A5C50170
class RACE_DefaultLogSwitchingStrategy : public RACE_LogSwitchingStrategy
{
    //##ModelId=3F72A5C6000C
  off_t m_Softlimit;
public:
    //##ModelId=3F72A5C6000E
  RACE_DefaultLogSwitchingStrategy(RACE_LogSwitcher *ls);
    //##ModelId=3F72A5C60011
  void InfoSnapshotWritten();
};
#endif

