#include<stdlib.h>
#include"RACE_LoggerQLS.h"
#include"envdebug.h"

envdebug *s_debug = new envdebug("RACE_LoggerQLS");

/*------------- The Logger Queue Level Strategy ------*/
/* The Default DescardDispensable Level is 7/8 = 0.875 of the Maximum */
RACE_LoggerQLS::RACE_LoggerQLS(const char *queuename):m_queuename(queuename),m_MaxLengthSum(1<<20),m_DiscardDispensableLengthSum(7*(1<<17)),m_loggerThreadId(0),m_relativeTimeout(1,0)
{
  const char *envstr;
  if((envstr=getenv("RACE_MAXQUEUELENGTH_KB"))!=NULL)
  {
    m_MaxLengthSum=atol(envstr)<<10;
    m_DiscardDispensableLengthSum=(m_MaxLengthSum>>3)*7;
  }
  if((envstr=getenv("RACE_DISCARDDISPENSIBLE_PERCENT"))!=NULL)
  {
    long percent=atol(envstr);
    m_DiscardDispensableLengthSum=m_MaxLengthSum/100*percent;
  }
  if((envstr=getenv("RACE_ENQUEUE_TIMEOUT_MS"))!=NULL)
  {
    long timeout = atol(envstr);
    m_relativeTimeout.msec(timeout);
  }
  s_debug->print("RACE_LoggerQLS max_queue.length()=%ld descardlength=%ld",m_MaxLengthSum,m_DiscardDispensableLengthSum);
}

const char *RACE_LoggerQLS::GetQueueName()
{
  return m_queuename;
}

bool RACE_LoggerQLS::is_full()
{
  /*
   * The Logger thread calls RACE_Dumpable::InformOffset().
   * This can generate Dumpables. The Loggerthread must't block when the queue
   * is full, since it is the only consumer
   * If the Logger Threds aslks us, the queue is never full.
   */
  return (cur_lengthsum_ >= m_MaxLengthSum) && (m_loggerThreadId != ACE_OS::thr_self());
}

bool RACE_LoggerQLS::can_enqueue()
{
  return (cur_lengthsum_ < m_MaxLengthSum)
       || m_loggerThreadId == (ACE_OS::thr_self());
}

bool RACE_LoggerQLS::is_critical_watermark_reached()
{
  return cur_lengthsum_ >= m_DiscardDispensableLengthSum;

}

ACE_Time_Value *RACE_LoggerQLS::GetCurrentEnqueueTimeout()
{
  m_currentAbsolutTimeout=ACE_OS::gettimeofday()+m_relativeTimeout;
  return &m_currentAbsolutTimeout;
}

