#include<output.h>
#include"RACE_Message_Block.h"
#include<iostream>
#include<string.h>
#include<stdlib.h>

//##ModelId=3F72A5C900B1
RACE_Data_Block* RACE_Data_Block::alloc(size_t size)
{
  RACE_Data_Block* ret=new RACE_Data_Block(size);
  RACE_COND_FAIL(ret==NULL,"%s%d:RACE_Data_Block::alloc: No More Memory",
            __FILE__,__LINE__);

  if(size>0)
  {
    ret->data_=(char *)malloc(size);
    RACE_COND_FAIL(ret->data_==NULL,
                      "%s%d:RACE_Data_Block::alloc: No More Memory",
                      __FILE__,__LINE__);
  }
  else
  {
    ret->data_=NULL;
  }
  return ret;
}

//##ModelId=3F72A5C900B7
RACE_Data_Block* RACE_Data_Block::clone()
{
  RACE_Data_Block* ret=new RACE_Data_Block(length());
  RACE_COND_FAIL(ret==NULL,"%s%d:RACE_Data_Block::alloc: No More Memory",
                    __FILE__,__LINE__);
  if(length()>0)
  {
    ret->data_=(char *)malloc(length());
    RACE_COND_FAIL(ret->data_==NULL,
                      "%s%d:RACE_Data_Block::alloc: No More Memory",
                      __FILE__,__LINE__);
    ret->cur_size_=length();
    memcpy(ret->data_,data_,length());
  }
  else
  {
    ret->data_=NULL;
  }
  return ret;
}

//##ModelId=3F72A5C900CC
RACE_Data_Block::~RACE_Data_Block()
{
  free(data_);
}

//##ModelId=3F72A5C900B9
RACE_Data_Block *RACE_Data_Block::duplicate()
{
  ACE_GUARD_RETURN(ACE_Thread_Mutex, ace_mon, this->refcountmutex_,0);
  reference_count_++;
  return this;
}
//##ModelId=3F72A5C900BB
void RACE_Data_Block::release()
{
  refcountmutex_.acquire();
  reference_count_--;
  if(reference_count_ == 0)
  {
    refcountmutex_.release();
    delete this;
  }
  else
  {
    refcountmutex_.release();
  }
}

//##ModelId=3F72A5C900BF
void RACE_Data_Block::grow(size_t n)
{
  RACE_COND_FAIL(n>freespace(),"RACE_Data_Block::grow: n=%d > freespace=d",
                    n,freespace());
  cur_size_+=n;
}

//##ModelId=3F72A5CA030C
int RACE_Message_Block::copy(const char *buf, size_t n)
{
  RACE_COND_FAIL(base()+size()<wr_ptr()+n,"RACE_Message_Block::copy:base()+size()<wr_ptr()+n");
  memcpy(wr_ptr(),buf,n);
  wr_ptr(n);
  return n;
}

//##ModelId=3F72A5CD0269
RACE_Message_Block *RACE_Message_Block::duplicate(void) const
{
  RACE_Message_Block *ret = CloneManagementData();
  ret->first_=first_;
  ret->db=db;
  ret->reset_current();
  for(RACE_Data_Block *cur=first_;cur!=NULL;cur=cur->next)
  {
    cur->duplicate();
  }
  return ret;
}

//##ModelId=3F72A5CD0326
RACE_Message_Block *RACE_Message_Block::clone(void) const
{
  RACE_Message_Block *ret = CloneManagementData();
  ret->block_size_=block_size_;
  ret->first_=NULL;
  RACE_Data_Block **cur_insert=&(ret->first_);
  for(RACE_Data_Block *cur=first_;cur!=NULL;cur=cur->next)
  {
    ret->db=*cur_insert = cur->clone();
    ret->reset_current();
    cur_insert=&((*cur_insert)->next);
  }
  return ret;
}

//##ModelId=3F72A5CE035D
RACE_Message_Block *RACE_Message_Block::CloneManagementData() const
{
  RACE_Message_Block *ret;
  ret = new RACE_Message_Block();
  RACE_COND_FAIL(ret==NULL,"%s%d:RACE_Message_Block::CloneManagementData(): No More Memory",
            __FILE__,__LINE__);
  return ret;
}

//##ModelId=3F72A5C902FE
RACE_Message_Block::RACE_Message_Block(size_t len):block_size_(len)
{
  first_=db=RACE_Data_Block::alloc(len);
  reset();
  next_=0;
  prev_=0;
}

//##ModelId=3F72A5C903DE
RACE_Message_Block::~RACE_Message_Block()
{
  RACE_Data_Block *temp=NULL;
  RACE_Data_Block *cur = first_;
  while(cur!=NULL)
  {
    temp=cur;
    cur=cur->next;
    temp->release();
  }
}

//##ModelId=3F72A5CD013D
void RACE_Message_Block::expand()
{
  db->next = RACE_Data_Block::alloc(block_size_);
  db=db->next;
  reset();
}

//##ModelId=3F72A5CC03B3
size_t RACE_Message_Block::total_length(void) const
{
  size_t ret=0;
  for(RACE_Data_Block *cur=first_;cur!=NULL;cur=cur->next)
  {
    ret += cur->length();
  }
  return ret;
}

//##ModelId=3F72A5CD0083
size_t RACE_Message_Block::total_size(void) const
{
  size_t ret=0;
  for(RACE_Data_Block *cur=first_;cur!=NULL;cur=cur->next)
  {
    ret += cur->size();
  }
  return ret;
}

#ifdef TEST_DATA_BLOCK
int main()
{
  std::cerr << "sizeof(RACE_Data_Block)=" << sizeof(RACE_Data_Block) << std::endl;
  RACE_Data_Block* rdb=RACE_Data_Block::alloc(4096);
  std::cerr << "maxsize=" << rdb->max_size() << std::endl;
}
#endif

