/**
 * counterfeit of the ACE_Message_Queue for
 * RACE_Message_Blocks
 */
#ifndef RACE_MESSAGE_QUEUE_H
#define RACE_MESSAGE_QUEUE_H
#include<ace/pre.h>
#include"RACE_Message_Block.h"


//##ModelId=3F72A5C501F5
//##Documentation
//##  @class ACE_Message_Queue_Base
//##
//##  @brief Workaround HP/C++ compiler bug with enums in templates.
//##
//##  The ever lamest HP/C++ compiler seems to fail if enums are
//##  defined inside a template, hence we have to move them into a
//##  base class.

class RACE_Message_Queue_Base
{
public:
  //##ModelId=3F72A5CF0165
  //##Documentation
  //## Default high and low water marks.
  enum
  {
    /// Default high watermark (16 K).
    DEFAULT_HWM = 16 * 1024,
    /// Default low watermark (same as high water mark).
    DEFAULT_LWM = 16 * 1024,
    /// Message queue was active before <activate> or <deactivate>.
    WAS_ACTIVE = 1,
    /// Message queue was inactive before <activate> or <deactivate>.
    WAS_INACTIVE = 2
  };


  //##ModelId=3F72A5CF0173
  RACE_Message_Queue_Base(void);

  //##ModelId=3F72A5CF0224
  //##Documentation
  //## Close down the message queue and release all resources.
  virtual int close(void) = 0;

  //##ModelId=3F72A5CF02D2
  virtual ~RACE_Message_Queue_Base(void);

  // = Enqueue and dequeue methods.

  // For the following enqueue and dequeue methods, the caller will
  // block until action is possible if <timeout> == 0.  Otherwise, it
  // will wait until the absolute time specified in *<timeout>
  // elapses.  These calls will -1 when queue is closed, deactivated
  // (in which case <errno> == <ESHUTDOWN>), when a signal occurs (in
  // which case <errno> == <EINTR>, or if the time specified in
  // timeout elapses (in which case <errno> == <EWOULDBLOCK>).

  /**
   * Retrieve the first <RACE_Message_Block> without removing it.  Note
   * that <timeout> uses <{absolute}> time rather than <{relative}>
   * time.  If the <timeout> elapses without receiving a message -1 is
   * returned and <errno> is set to <EWOULDBLOCK>.  If the queue is
   * deactivated -1 is returned and <errno> is set to <ESHUTDOWN>.
   * Otherwise, returns -1 on failure, else the number of items still
   * on the queue.
   */
  //virtual int peek_dequeue_head(RACE_Message_Block *&first_item, ACE_Time_Value *timeout = 0) = 0;


  //##ModelId=3F72A5CF0382
  //##Documentation
  //## Enqueue a <RACE_Message_Block *> into the tail of the queue.
  //## Returns number of items in queue if the call succeeds or -1
  //## otherwise.
  //## The Timeout is set over the RACE_QueuelevelStrategy
  virtual int enqueue(RACE_Message_Block *new_item) = 0;

//##ModelId=3F72A5D00066
//##Documentation
//##  Dequeue and return the <ACE_Message_Block *> at the head of the
//##  queue.  Returns number of items in queue if the call succeeds or
//##  -1 otherwise.
//## The Timeout is set over the RACE_QueuelevelStrategy
  virtual int dequeue (RACE_Message_Block *&first_item) = 0;

  // = Check if queue is full/empty.
//##ModelId=3F72A5D00153
//##Documentation
//## True if queue is full, else false.//## True if queue is empty, else false.
  virtual int is_full (void) = 0;
    //##ModelId=3F72A5D0021F
  virtual int is_empty (void) = 0;

   // = Queue statistic methods.
//##ModelId=3F72A5D002D1
//##Documentation
//##  Number of total bytes on the queue, i.e., sum of the message
//##  block sizes.
  virtual size_t message_bytes (void) = 0;

 //##ModelId=3F72A5D00383
 //##Documentation
 //##  Number of total bytes on the queue, i.e., sum of the message
 //##  block sizes.
  virtual size_t message_length (void) = 0;

//##ModelId=3F72A5D1004B
//##Documentation
//##  Number of total messages on the queue.
  virtual size_t message_count (void) = 0;

private:
  // = Disallow these operations.
  ACE_UNIMPLEMENTED_FUNC (void operator= (const RACE_Message_Queue_Base &))
  ACE_UNIMPLEMENTED_FUNC (RACE_Message_Queue_Base (const RACE_Message_Queue_Base &))
};
#if defined (__ACE_INLINE__)
#include "RACE_Message_Queue.i"
#endif /* __ACE_INLINE__ */

#include <ace/post.h>
#endif /* ACE_MESSAGE_QUEUE_H */
