#ifndef RACESNAPSHOTTRIGGER_H
#define RACESNAPSHOTTRIGGER_H
#include<ace/Synch.h>

//##ModelId=3FB37BB503D2
//##Documentation
//## A simple Queue for managing Snapshots
class RACE_SnapshotQueue
{
public:
    //##ModelId=3FB37BB503E1
  class Entry
  {
  public:
        //##ModelId=3FB37BB603A8
    Entry(time_t timestamp,int pending)
      :m_timestamp(timestamp),m_pending(pending),mp_next(0)
    {
    }
        //##ModelId=3FB37BB603B2
    virtual ~Entry();
        //##ModelId=3FB37BB603B4
    time_t GetTimestamp() { return m_timestamp; }
        //##ModelId=3FB37BB603B5
    int GetPending() { return m_pending; }
        //##ModelId=3FB37BB603B6
    int DecrPending() { return --m_pending; }
        //##ModelId=3FB37BB603B7
    virtual void Log()=0;
  private:
        //##ModelId=3FB37BB60399
    time_t m_timestamp;
        //##ModelId=3FB37BB6039A
    int  m_pending;
        //##ModelId=3F16A48F00F0
    Entry *mp_next;
    friend class ::RACE_SnapshotQueue;
  };
    //##ModelId=3FB37BB603C7
  RACE_SnapshotQueue():mp_first(0),mpp_last(&mp_first) {}
    //##ModelId=3FB37BB603C8
  virtual ~RACE_SnapshotQueue();
    //##ModelId=3FB37BB603CA
  void GetCountAndSum(int& count,int& sumpending);
    //##ModelId=3FB37BB603D3
  void enqueue(Entry *e);
    //##ModelId=3FB37BB603D5
  bool IsEmpty() { return mp_first==0; }
    //##ModelId=3FB37BB603D6
  time_t GetTopTimestamp() { return mp_first->GetTimestamp(); }
    //##ModelId=3FB37BB603D7
  int    GetTopPending()   { return mp_first->GetPending(); }
    //##ModelId=3FB37BB603D8
  int    DecrTopPending()  { return mp_first->DecrPending(); }
    //##ModelId=3FB37BB603D9
  Entry *dequeue();

    //##ModelId=3FB37BB603E1
  Entry *GetTop() { return mp_first; }

    //##ModelId=3FB37BB603E2
  //##Documentation
  //## gets the entry with timestamp from the queue
  //## Returns NULL if no entry with exactly matching Timestamp
  Entry *GetEntry(time_t timestamp);
protected:
    //##ModelId=3FB37BB603E4
  Entry *GenNewEntry(time_t timestamp,int pending);
private:
    //##ModelId=3F16A48F0113
  Entry *mp_first;
    //##ModelId=3F16A48F011D
  Entry **mpp_last;
};

const int ST_UNKOWN=-1;

//##ModelId=3FB37BB503C2
//##Documentation
//##  Interface for creation and Collecting Snapshots,
//##  used by RACE_SnapshotTrigger
class RACE_SnapshotCreator
{
public:
    //##ModelId=3FB37BB60384
  //##Documentation
  //##  has to return the total number of Dumpables pending
  //##  returns ST_UNKOWN if not known
  //##  The default-Implemantation does not know and returns ST_UNKOWN.
  //##  For example, for OnEventSnapshots it is known.
  virtual int GetPendingTotal();
    //##ModelId=3FB37BB60386
  //##Documentation
  //## has to return the number of Dumpables pending for the current
  //## Snapshot
  //## returns ST_UNKOWN if not known
  //##  The default-Implementation does not know and returns ST_UNKOWN.
  //## In WholeSnapshots it is known
  virtual int GetPendingForYou();
    //##ModelId=3FB37BB60388
  virtual RACE_SnapshotQueue::Entry *GenNewEntry(time_t timestamp,int pending)=0;
    //##ModelId=3FB37BB60395
  virtual ~RACE_SnapshotCreator();
};

//##ModelId=3FB37BB503E2
//##Documentation
//##  Triggers the logging of a snapshot if all pending Dumpables
//##  are written.
//##  Thus the not written Dumpables are referenced in a Snapshot
//##  This is a strategy-Class
//##
//##  PrepareSnapshot() is called from the Main-Thread,
//##  InformOffset is called from the Loggerthread, thus we need Koordination
class RACE_SnapshotTrigger
{
public:
  //##ModelId=3FB37BB70028
  RACE_SnapshotTrigger(RACE_SnapshotCreator *sc,const char *description,RACE_SnapshotQueue *sq=NULL);
    //##ModelId=3FB37BB7002C
  virtual ~RACE_SnapshotTrigger();

  //##ModelId=3FB37BB7002E
  void PrepareSnapshot(time_t timestamp);

  //##ModelId=3FB37BB70038
  //##Documentation
  //##  This is called in the InformOffset()-Callback when the Dumpable
  //##  is written or discarded
  //##  In the view of the Snapshottrigger, there is no difference
  //##  between Inserting and Discarding
  void InformInsert();

    //##ModelId=3FB37BB70039
  bool IsSnapshotPending() { return !m_queue->IsEmpty(); }
    //##ModelId=3FB37BB7003A
  RACE_SnapshotQueue::Entry *GetCurrent()     { return m_queue->GetTop(); }

    //##ModelId=3FB37BB7003B
  //##Documentation
  //##  This is called in the InformOffset()-Callback when the Dumpable
  //##  is written
  //##  it allows a Insertion overtaking an other in the case of jumbling
  void InformInsert(time_t timestamp);

    //##ModelId=3FB37BB7003D
  //##Documentation
  //## Getting the Entry from the Queue
  //## Returns 0 if not found
  RACE_SnapshotQueue::Entry *GetSnapshot(time_t timestamp) { return m_queue->GetEntry(timestamp); }
private:
    //##ModelId=3EFAEA120270
  RACE_SnapshotCreator *mp_snapcreator;

    //##ModelId=3EFAEA12029D
  ACE_Thread_Mutex m_mutex;

    //##ModelId=3FB37BB70019
  //##Documentation
  //## Description for Debugging
  const char *m_description;

    //##ModelId=3FB37BB7001A
  //##Documentation
  //## last timestamp for controlling rthe monotonie of the prepare - Calls
  time_t m_lastTimestamp;
    //##ModelId=3F16A48F01A8
  RACE_SnapshotQueue *m_queue;
};
#endif

