#ifndef RACE_TASK_T_H
#define RACE_TASK_T_H
/**
 * @file RACE_Task_T.h
 * Counterfeit of ACE_Task_T adaped to the RACE_Message_Queue
 */
#include<ace/pre.h>
#if !defined (ACE_LACKS_PRAGMA_ONCE)
# pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */
#include"RACE_Message_Queue_T.h"
#include<ace/Synch_T.h>
#include<ace/Task.h>

//##ModelId=3F72A5C50222
//##Documentation
//##  @class RACE_Task
//##
//##  Implements the Active Object Pattern

template <ACE_SYNCH_DECL>
class RACE_Task : public ACE_Task_Base
{
 public:
    // = Initialization/termination methods.
//##ModelId=3F72A5D503E1
//##Documentation
//##  Initialize a Task, supplying a thread manager and a message
//##  queue.  If the user doesn't supply a ACE_Message_Queue pointer
//##  then we'll allocate one dynamically.  Otherwise, we'll use the
//##  one passed as a parameter.

  RACE_Task(RACE_QueueLevelStrategy *qls,ACE_Thread_Manager *thr_mgr = 0)
    : ACE_Task_Base(thr_mgr),msg_queue_(qls)
  {
  }

//##ModelId=3F72A5D60002
//##Documentation
//## Destructor.
  virtual ~RACE_Task(void)
    {
      msg_queue()->close();
    }

    //##ModelId=3F7308BB0328
  //##Documentation
  //## Typedef for the message queue used by RACE_Task.
  typedef RACE_Message_Queue<ACE_SYNCH_USE> Message_Queue_t;

//##ModelId=3F72A5D60008
//##Documentation
//## Gets the message queue associated with this task.
  Message_Queue_t *msg_queue(void) { return &msg_queue_; }

 protected:
//##ModelId=3F72A5D503CD
//##Documentation
//## Queue of messages on the ACE_Task..
  Message_Queue_t msg_queue_;
    //##ModelId=3F72A5D6000D
  int getq(RACE_Message_Block* &m)
    {
      return msg_queue()->dequeue(m);
    }

 private:
  // = Disallow these operations.
  ACE_UNIMPLEMENTED_FUNC (void operator= (const RACE_Task<ACE_SYNCH_USE> &))
  ACE_UNIMPLEMENTED_FUNC (RACE_Task (const ACE_Task<ACE_SYNCH_USE> &))

};

#include <ace/post.h>
#endif /* RACE_TASK_T_H */
