#include "ActiveConsumer.h"
#include "RACE_CmdDumpable.h"
#include <time.h>
#include <iostream>

envdebug *ActiveConsumer::ms_debug = new envdebug("ActiveConsumer");

int ActiveConsumer::DoWork(RACE_Dumpable *dumpable)
{
  if(m_canceled && dumpable->length() > 0 && !m_qls.IsCallerLoggerThread())
  {
    output::warn("ActiveConsumer::log: Logger canceled - discarding dumpable");    dumpable->Discard();
    dumpable->release();
    return -1;
  }
  int r=msg_queue()->enqueue(dumpable);
  ms_debug->print("ActiveConsumer::log enqueue returns %d",r);
  if(r<0 && errno==EWOULDBLOCK)
  {
    output::error("EnQueuing TIMEOUT - Canceling");
    Cancel();
    // Cancel Imediately, don't wait until the Canceldumpable reaches the logger
    m_canceled=true;
    return -1;
  }
  output::cond_fail(r<0,"Logger::log enqueue");
  return 0;
}

static RACE_LogfileOffset currentoffset(1,0);

int ActiveConsumer::svc(void)
{
  int count=1;
  RACE_Message_Block *mb;
  m_qls.SetLoggerThreadId(ACE_OS::thr_self());
  for(;!(m_canceled && msg_queue()->is_empty()); count++)
  {
    ms_debug->print("(%t) in iteration %d before getq()\n",count);
    if (this->getq(mb) == -1)
    {
      output::error("(%t) in iteration %d, got result -1, exiting\n",count);
      break;
    }
    RACE_Dumpable *d=ACE_dynamic_cast(RACE_Dumpable *,mb);
    int length = d->total_length();

    if (length > 0)
    {
      ms_debug->print("(%t) in iteration %d, length = %d fragmented=%s\n",
                      count,length,(d->IsFragmented() ? "true" : "false"));
      if(d->IsDiscardable() && m_qls.is_critical_watermark_reached())
      {
        output::warn("ActiveConsumer: Discarding dispensable Dumpable");
        d->Discard();
      }
      else
      {
        /* Do work */
        struct timespec ts;
        ts.tv_sec=0;
        ts.tv_nsec=d->total_length()<<10;
        std::cerr << "Work begin time=" << ts.tv_nsec << std::endl;
        nanosleep(&ts,NULL);
        currentoffset.offset+=d->total_length();
        d->InformOffset(currentoffset);
        std::cerr << "End Work" << std::endl;
      }
    }
    else
    {
      if(ACE_dynamic_cast(RACE_CancelCmdDumpable *,d))
      {
          ms_debug->print("ActiveConsumer: RACE_CancelCmdDumpable");
          m_canceled = true; // dissallow queueing
      }
      else
      {
         output::error("ActiveConsumer received an Dumpable with length=%d, queuelength=",length,msg_queue()->message_count());
      }
      d->release();
    }
  }
  return 0;
}

void ActiveConsumer::Cancel()
{
  RACE_Dumpable *d = new RACE_CancelCmdDumpable();
  /*
   * In all cases, the CancesDumbable must be enqueued
   * The CActiveObject mus be possible to flush its queue and so on
   */
  int r=msg_queue()->enqueue_force(d);
  ms_debug->print("ActiveConsumer::log enqueue_force returns %d",r);
  output::cond_error(r<0,"ActiveConsumer::enqueue_force");
}

ActiveConsumer* ActiveConsumer::Duplicate()
{
  m_refcount++;
  ms_debug->print("ActiveConsumer::duplicate: %d References",m_refcount);
  return this;
}

void ActiveConsumer::Release()
{
  ms_debug->print("ActiveConsumer::release: %d ",m_refcount);
  m_refcount--;
  if(m_refcount == 0)
  {
    ms_debug->print("ActiveConsumer::release: would delete myself: thr_count=%d",
                    thr_count());
  }
}
