#include<time.h>
#include"ActiveConsumer.h"
#include<iostream>
using namespace std;

//##Documentation
//## simmulates a LoggerSnapshotConnection in queuingtest
//## it keeps all the Data for feeding back a Dumpable
class CFeedBackConnection
{
public:
  CFeedBackConnection(ActiveConsumer *ac):mp_consumer(ac) {}
  void feedback(char pat);
private:
  ActiveConsumer *mp_consumer;
};

class CQTestDumpable : public RACE_Dumpable
{
public:
  CQTestDumpable(size_t s,char pat,bool discardable)
    :RACE_Dumpable(s),m_pattern(pat),m_discardable(discardable),mp_fbc(0)
  {
    memset(wr_ptr(),pat,s);
    wr_ptr(s);
  }
  void SetFeedback(CFeedBackConnection *fbc) { mp_fbc=fbc; }
  virtual RACE_Message_Block *CloneManagementData() const;
  virtual void InformOffset(RACE_LogfileOffset offset);
  virtual bool IsDiscardable();
  virtual void Discard();
private:
  char m_pattern;
  bool m_discardable;
  CFeedBackConnection *mp_fbc;
  //##Documentation
  //## For Cloning
  CQTestDumpable(char pat,bool disc)
    :m_pattern(pat),m_discardable(disc),mp_fbc(0) {}
};

void CQTestDumpable::InformOffset(RACE_LogfileOffset offset)
{
  cerr << "Written Pattern '" << m_pattern << "' with length=" << total_length()
       << "\tat offset " << offset.part << ':' << offset.offset << endl;
  if(mp_fbc)
  {
    mp_fbc->feedback(m_pattern);
  }
}

void CQTestDumpable::Discard()
{
  cerr << "Discarding Dumpable with Pattern '" << m_pattern << "' and length="
       << total_length() << endl;
}

bool CQTestDumpable::IsDiscardable()
{
  return m_discardable;
}

//##Documentation
//## The InformOffset-Method blocks the CActiveObject a longtime
//## for simulating TIMEOUT
class CQBlockingDumpable : public RACE_Dumpable
{
public:
  CQBlockingDumpable():RACE_Dumpable(10)
  {
    memcpy(wr_ptr(),"BLOCKING",8);
    wr_ptr(8);
  }
  virtual void InformOffset(RACE_LogfileOffset offset);
};

void CQBlockingDumpable::InformOffset(RACE_LogfileOffset)
{
  struct timespec ts;
  ts.tv_sec=8;
  ts.tv_nsec=0;
  cerr << "CQBlockingDumpable::InformOffset: I need "
       << ts.tv_sec << "s" << endl;
  nanosleep(&ts,NULL);
}

CQTestDumpable *genentry(size_t s,char pat,bool discardable)
{
  return new CQTestDumpable(s,pat,discardable);
}

CQTestDumpable *genfragmented(size_t s,int count,char pat1,char pat2,bool discardable)
{
  int i;
  CQTestDumpable *f = new CQTestDumpable(s,pat1,discardable);
  for(i=0;i<count-1;i++)
  {
    f->expand();
    memset(f->wr_ptr(),pat2,s);
    f->wr_ptr(s);
  }
  return f;
}

RACE_Message_Block *CQTestDumpable::CloneManagementData() const
{
  CQTestDumpable* ret = new CQTestDumpable(m_pattern,m_discardable);
  return ret;
}

void CFeedBackConnection::feedback(char pat)
{
  CQTestDumpable *fd;
  cerr << "Feedback from '" << pat << "' - Dumpable with length 200" << endl;
  fd=genfragmented(100,2,pat,pat,false);
  mp_consumer->DoWork(fd);
}

static int s_blockingtest=0;
static CFeedBackConnection *s_fbc=NULL;
void simmulator(ActiveConsumer *ac)
{
  int r;
  int g=0;
  for(int n=0;n<1000;n++)
  {
    CQTestDumpable *d;
    bool discardable = (n%2==0);
    // Allocate a new message.
    if(g<6)
    {
      size_t s= (1<<(g+8));
      char pat='A'+(n%26);
      d = genentry(s,pat,discardable);
      cerr << "Sender: iteration " << n << " g= " << g
           << " size=" << s << " Pattern " << pat
           << (discardable ? " discardable " : "")
           << endl;
    }
    else
    {
      size_t s=4096;
      char pat1='A'+(n%26);
      char pat2='a'+(n%26);
      d = genfragmented(s,g-4,pat1,pat2,discardable);
      cerr << "Sender: iteration " << n << " g= " << g
           << " Fragmented: " << g-4 << "* size=" << s << " Pattern "
           << pat1 << "/" << pat2
           << (discardable ? " discardable " : "")
           << endl;
    }
    if(n%9==0)
    {
      cerr << "Setting Feedback ... " << endl;
      d->SetFeedback(s_fbc);
    }
    r=ac->DoWork(d);
    if(r<0)
    {
      cerr << "Ok, canceling the production of dumpables" << endl;
      break;
    }
    g=(g+1)%10;
    if(n==500 && s_blockingtest!=0)
    {
      RACE_Dumpable *bd = new CQBlockingDumpable();
      cerr << "Queuing Blocking Dumpable" << endl;
      ac->DoWork(bd);
    }
  }
  cerr << "End of Simulation" << endl;
  ac->Cancel();
}

int main(int argc,char**argv)
{
  if(argc>1 && strcmp(argv[1],"-b")==0)
  {
    s_blockingtest=1;
  }
  ActiveConsumer *ac=new ActiveConsumer();
  ac->activate();
  s_fbc=new CFeedBackConnection(ac);
  simmulator(ac);

  ac->Release();
  ACE_Thread_Manager::instance()->wait();
  delete s_fbc; s_fbc=NULL;
  std::cerr << "done" << std::endl;
  return 0;
}
