#include <string>
#include <cstdlib>
#include <cstdio>
#include <cstdarg>

#include "envdebug.h"
#include "env_access.h"
#include "output.h"

envdebug::envdebug (const char *classname)
{
  // Example: for a classname "RtecSupplier",
  // an environment variable DBG_RtecSupplier
  // has to be defined to switch on debug messages.
  env_level dbg_all_flag ("DBG_ALL");
  if (dbg_all_flag.is_on ())
    _dbg = true;
  else {
    const char *prefix = "DBG_";
    char env_var[strlen(classname) + strlen(prefix) + 1];
    sprintf (env_var, "%s%s", prefix, classname);
    env_level dbg_special_flag (env_var);
    _dbg = dbg_special_flag.is_on ();
  }
  env_flag no_timestamps_flag ("DBG_NO_TIMESTAMPS");
  if (! no_timestamps_flag.is_on ())
    output::time_output_on ();
}

void envdebug::print (char *msg, ...)
{
  if (! _dbg)
    return;
  output::cond_fail (msg == NULL, "envdebug::print got NULL msg");
  va_list vargu;
  char str[1024];
  va_start (vargu, msg);
  vsprintf (str, msg, vargu);
  va_end (vargu);
  output::component::get_default_component ().general_output_function (output::e_debug, str);
}

envdebug::component::component (const envdebug* pe, const char *name)
  : m_pe(pe), m_oc(name)
{
  output::cond_fail (m_pe == NULL, "envdebug::component::component: no envdebug pointer given");
}

void envdebug::component::print (char *msg, ...)
{
  if (! m_pe->is_on ())
    return;
  output::cond_fail (msg == NULL, "envdebug::print got NULL msg");
  va_list vargu;
  char str[1024];
  va_start (vargu, msg);
  vsprintf (str, msg, vargu);
  va_end (vargu);
  m_oc.general_output_function (output::e_debug, str);
}

