#ifndef __envdebug_h__
#define __envdebug_h__

#include "output.h"

/*
 * $Id:$
 */

//##Documentation
//## Conditionalizes debug output based on environment symbol settings.
class envdebug {
private:
   bool _dbg;
public:
   //##Documentation
   //## Example: For a classname "RtecSupplier", an environment variable
   //## DBG_RtecSupplier has to be defined to switch on debug messages.
   //## The value of the symbol shall not be empty, "0", or "off".
   //## All other values switch the debug output on.
   envdebug (const char *classname);
   virtual ~envdebug () {}

   bool is_on () const { return _dbg; }

   void print (char *msg, ...);

    //##Documentation
    //## Usage:
    //## In a method Class::Method(), declare a local variable
    //## as follows:
    //##   envdebug::component c("Class::Method");
    //## (The name, `c', can be any name.)
    //## The local object will be automatically destructed upon
    //## exit from the method.
    class component {
    public:
      component (const envdebug *pe, const char *name);
      virtual ~component() {}

      void print (char *msg, ...);

    private:
      component();
      component(const component&);
      component& operator=(const component&);
      const envdebug *m_pe;
      output::component m_oc;
    };
};

#endif    // ifndef __envdebug_h__

