#ifndef OUTPUT_H
#define OUTPUT_H
/*
 * $Id:  $
 */
#include <cstdarg>
#include <string>

// declaration of type bool needed by the stupid DEC cxx
#if defined (__DECCXX) && defined (THE_OLDER_CXX)
# define bool int
# define true 1
# define false 0
#endif

//##Documentation
//## This utility is used for formatting and printing different kinds of
//## messages of the form
//##  <executable_name>:: [<component_name>] <message_type>: <message>
//##

namespace output
{
    //##Documentation
    //## Optional method to establish a special logging capability.
    //## Some logging mechanisms support optional service initialization
    //## for the usage of special channels, levels, etc.
    //## set_alternative_logging_capability sets those parameters to
    //## pre-defined values.
    void set_alternative_logging_capability ();

    void debug (char *msg, ...);
    void message (char *msg, ...);
    void warn (char *msg, ...);
    void error (char *msg, ...);
    void report (char *msg, ...);
    void fail (char *msg, ...);

    void cond_debug (bool validity, char *msg, ...);
    void cond_message (bool validity, char *msg, ...);
    void cond_warn (bool validity, char *msg, ...);
    void cond_error (bool validity, char *msg, ...);
    void cond_fail (bool validity, char *msg, ...);

    void dump_hex (const unsigned char *buffer, unsigned buffer_len, unsigned bytes_per_line = 30);

    //##Documentation
    //## Set the executable name. This appears first in the output.
    void set_executable_name (const char *name);

    //##Documentation
    //## Set the programming language. This is output directly after
    //## the executable name.
    //## Default: C++.
    void set_language (const char *name);

    //##Documentation
    //## Set the component name. This is optional.
    //## If given, it is output after the programming language.
    void set_component_name (const char *name);

    //##Documentation
    //## Return the executable name that was set by `set_executable_name'.
    const char *get_executable_name ();

    //##Documentation
    //## Switch on time stamps in the output.
    void time_output_on ();
    //##Documentation
    //## Timestamp output is off by default.
    void time_output_off ();

    //##Documentation
    //## The message type enum.
    //## Not required by the user interface, but made visible here
    //## because other utilities might need it.
    enum output_classification { e_debug, e_message, e_warn, e_error, e_fail, e_unspecific };

    //##Documentation
    //## Usage:
    //## In a method Class::Method(), declare a local variable
    //## as follows:
    //##   output::component c("Class::Method");
    //## (The name, `c', can be any name.)
    //## The local object will be automatically destructed upon
    //## exit from the method.
    class component {
    public:
      component (const char *name);
      virtual ~component() {}

      void debug (char *msg, ...);
      void message (char *msg, ...);
      void warn (char *msg, ...);
      void error (char *msg, ...);
      void report (char *msg, ...);
      void fail (char *msg, ...);

      void cond_debug (bool validity, char *msg, ...);
      void cond_message (bool validity, char *msg, ...);
      void cond_warn (bool validity, char *msg, ...);
      void cond_error (bool validity, char *msg, ...);
      void cond_fail (bool validity, char *msg, ...);

      void dump_hex (const unsigned char *buffer, unsigned buffer_len, unsigned bytes_per_line = 30);

      void general_output_function (output_classification kindof, char *str);

      static component& get_default_component () { return the_default_component; }

    private:
      component();
      component(const component&);
      component& operator=(const component&);
      std::string m_name;
      static component the_default_component;
    };
};

#define RACE_COND_FAIL(COND,MESSAGE...) output::cond_fail(COND,MESSAGE)
#define RACE_WARN(MESSAGE...)           output::warn(MESSAGE)
#define RACE_ERROR(MESSAGE...)          output::error(MESSAGE)

#endif /* OUTPUT_H */
